/* A servlet to display the contents of the PostgreSQL mydatabasename database */

import java.io.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ElectroDWH extends HttpServlet
{
    public String getServletInfo() {
       return "Servlet connects to PostgreSQL database and displays result of a SELECT";
    }

    private Connection dbcon;  // Connection

    // "init" sets up a database connection
    public void init(ServletConfig config) throws ServletException {
        String loginUser = "postgres";
        String loginPasswd = "SQL";
        String loginUrl = "jdbc:postgresql://localhost:5432/electroDWH";

        // Load the PostgreSQL driver
        try {
            Class.forName("org.postgresql.Driver");
            dbcon = DriverManager.getConnection(loginUrl, loginUser, loginPasswd);
        } catch (ClassNotFoundException ex) {
            System.err.println("ClassNotFoundException: " + ex.getMessage());
            throw new ServletException("Class not found Error");
        } catch (SQLException ex) {
            System.err.println("SQLException: " + ex.getMessage());
        }
    }

    // Use http GET
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {
        response.setContentType("text/html");    // Response mime type

        // Output stream to STDOUT
        PrintWriter out = response.getWriter();

        out.println("<HTML><Head><Title>ElectroDWH</Title></Head>");

        try {
            String CONSULTA = request.getParameter("consulta");
            if (CONSULTA != null) {

               // Declare our statement
               Statement statement = dbcon.createStatement();
               out.println("<table border>");

               if (CONSULTA.equals("fets")) {
                   out.println("<Body><H2>Consulta taula fets</H2>");

                   String query = "SELECT DWH_res_prod, DWH_res_accio,";
                   query +=       "       DWH_res_incid";
                   query +=       "  FROM DWH_res";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>producte</td>" +
                        "<td>incidencia</td>" +
                        "<td>accio</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_prd = rs.getString("DWH_res_prod");
                       String res_inc = rs.getString("DWH_res_incid");
                       String res_acc = rs.getString("DWH_res_accio");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td><A href=\"./Electrodomestics?consulta=productes" +
                                 "&codi=" + res_prd + "\">" + res_prd +
                                 "</A></td>" +
                            "<td><A href=\"./Electrodomestics?consulta=incidencies" +
                                 "&codi=" + res_inc + "\">" + res_inc +
                                 "</A></td>" +
                            "<td><A href=\"./Electrodomestics?consulta=accions" +
                                 "&codi=" + res_acc + "\">" + res_acc +
                                 "</A></td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_1")) {
                   out.println("<Body><H2>Nombre incidncies per tipus d'incidncia</H2>");

                   String query = "SELECT DWH_inc_tipus AS Tipus_incidencia,";
                   query +=       "       count(distinct DWH_res_incid) as Nombre";
                   query +=       "  FROM DWH_res, DWH_inc";
                   query +=       " WHERE DWH_res_incid = DWH_inc_codi";
                   query +=       " GROUP BY DWH_inc_tipus";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Tipus incidncia</td>" +
                        "<td>Nombre</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_tip_inc = rs.getString("Tipus_incidencia");
                       String res_tip_num = rs.getString("Nombre");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_tip_inc + "</td>" +
                            "<td align=\"right\">" + res_tip_num + "</td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_2")) {
                   out.println("<Body><H2>Nombre incidncies per tipus de producte</H2>");

                   String query = "SELECT DWH_prd_tipus AS Tipus_producte,";
                   query +=       "       count(distinct DWH_res_incid) as Nombre";
                   query +=       "  FROM DWH_res, DWH_prd";
                   query +=       " WHERE DWH_res_prod = DWH_prd_codi";
                   query +=       " GROUP BY DWH_prd_tipus";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Tipus producte</td>" +
                        "<td>Nombre</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_tip_prd = rs.getString("Tipus_producte");
                       String res_tip_num = rs.getString("Nombre");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_tip_prd + "</td>" +
                            "<td align=\"right\">" + res_tip_num + "</td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_3")) {
                   out.println("<Body><H2>Cost garanties per tipus d'incidncia</H2>");

                   String query = "SELECT DWH_inc_tipus AS Tipus_incidencia,";
                   query +=       "       sum(DWH_acc_cost) as Cost_garantia";
                   query +=       "  FROM DWH_res, DWH_inc, DWH_acc, DWH_prd";
                   query +=       " WHERE DWH_res_incid = DWH_inc_codi";
                   query +=       "   AND DWH_res_accio = DWH_acc_codi";
                   query +=       "   AND DWH_res_prod = DWH_prd_codi";
                   query +=       "   AND DWH_inc_ingar = 'S'";
                   query +=       "   AND DWH_inc_dtIni <= (DWH_prd_data + (DWH_prd_tmgar * interval '1 year'))";
                   query +=       " GROUP BY DWH_inc_tipus";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Tipus incidncia</td>" +
                        "<td>Cost garantia</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_tip_inc = rs.getString("Tipus_incidencia");
                       String res_tip_cst = rs.getString("Cost_garantia");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_tip_inc + "</td>" +
                            "<td align=\"right\">" + res_tip_cst + "</td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_4")) {
                   out.println("<Body><H2>Cost garanties per tipus de producte</H2>");

                   String query = "SELECT DWH_prd_tipus as Tipus_producte,";
                   query +=       "       sum(DWH_acc_cost) as Cost_garantia";
                   query +=       "  FROM DWH_res, DWH_inc, DWH_acc, DWH_prd";
                   query +=       " WHERE DWH_res_incid = DWH_inc_codi";
                   query +=       "   AND DWH_res_accio = DWH_acc_codi";
                   query +=       "   AND DWH_res_prod = DWH_prd_codi";
                   query +=       "   AND DWH_inc_ingar = 'S'";
                   query +=       "   AND DWH_inc_dtIni <= (DWH_prd_data + (DWH_prd_tmgar * interval '1 year'))";
                   query +=       " GROUP BY DWH_prd_tipus";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Tipus producte</td>" +
                        "<td>Cost garantia</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_tip_prd = rs.getString("Tipus_producte");
                       String res_tip_cst = rs.getString("Cost_garantia");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_tip_prd + "</td>" +
                            "<td align=\"right\">" + res_tip_cst + "</td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_5")) {
                   out.println("<Body><H2>Cost de les incidncies en un determinat perode</H2>");

                   String DATAINICI = request.getParameter("FormDataInici");
                   if (DATAINICI == null || DATAINICI == "" || !isValidDate(DATAINICI)) {
                       out.println("FormDataInici = '" + DATAINICI + "' no s correcte, assumim 2001-01-01");
                       DATAINICI = "2001-01-01";
                   }

                   String DATAFINAL = request.getParameter("FormDataFinal");
                   if (DATAFINAL == null || DATAFINAL == "" || !isValidDate(DATAFINAL)) {
                       out.println("FormDataFinal = '" + DATAFINAL + "' no s correcte, assumim 2011-12-31");
                       DATAFINAL = "2011-12-31";
                   }

                   String query = "SELECT 'De " + DATAINICI + " a " + DATAFINAL + "'::text as periode,";
                   query +=       "       sum(DWH_acc_cost) as Cost_incidencies";
                   query +=       "  FROM DWH_res, DWH_inc, DWH_acc, DWH_prd";
                   query +=       " WHERE DWH_res_incid = DWH_inc_codi";
                   query +=       "   AND DWH_res_accio = DWH_acc_codi";
                   query +=       "   AND DWH_res_prod = DWH_prd_codi";
                   query +=       "   AND DWH_inc_dtIni >= to_date('" + DATAINICI + "','YYYY-MM-DD')";
                   query +=       "   AND DWH_inc_dtFi <= to_date('" + DATAFINAL + "','YYYY-MM-DD');";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Perode</td>" +
                        "<td>Cost incidncies</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_inc_per = rs.getString("periode");
                       String res_inc_cst = rs.getString("Cost_incidencies");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_inc_per + "</td>" +
                            "<td align=\"right\">" + res_inc_cst + "</td>" +
                            "</tr>");
                   }
               } else if (CONSULTA.equals("informe_6")) {
                   out.println("<Body><H2>Cost de les garanties en un determinat perode</H2>");

                   String DATAINICI = request.getParameter("FormDataInici");
                   if (DATAINICI == null || DATAINICI == "" || !isValidDate(DATAINICI)) {
                       out.println("FormDataInici = '" + DATAINICI + "' no s correcte, assumim 2001-01-01");
                       DATAINICI = "2001-01-01";
                   }

                   String DATAFINAL = request.getParameter("FormDataFinal");
                   if (DATAFINAL == null || DATAFINAL == "" || !isValidDate(DATAFINAL)) {
                       out.println("FormDataFinal = '" + DATAFINAL + "' no s correcte, assumim 2011-12-31");
                       DATAFINAL = "2011-12-31";
                   }

                   String query = "SELECT 'De " + DATAINICI + " a " + DATAFINAL + "'::text as periode,";
                   query +=       "       sum(DWH_acc_cost) as Cost_garantia";
                   query +=       "  FROM DWH_res, DWH_inc, DWH_acc, DWH_prd";
                   query +=       " WHERE DWH_res_incid = DWH_inc_codi";
                   query +=       "   AND DWH_res_accio = DWH_acc_codi";
                   query +=       "   AND DWH_res_prod = DWH_prd_codi";
                   query +=       "   AND DWH_inc_ingar = 'S'";
                   query +=       "   AND DWH_inc_dtIni <= (DWH_prd_data + (DWH_prd_tmgar * interval '1 year'))";
                   query +=       "   AND DWH_inc_dtIni >= to_date('" + DATAINICI + "','YYYY-MM-DD')";
                   query +=       "   AND DWH_inc_dtFi <= to_date('" + DATAFINAL + "','YYYY-MM-DD');";

                   out.println("<tr BGCOLOR=\"#80FFCC\">" +
                        "<td>Perode</td>" +
                        "<td>Cost garanties</td>" +
                        "</tr>");

                   // Perform the query
                   ResultSet rs = statement.executeQuery(query);

                   // Iterate through each row of rs
                   while (rs.next()) {
                       String res_inc_per = rs.getString("periode");
                       String res_inc_cst = rs.getString("Cost_garantia");
                       out.println("<tr BGCOLOR=\"#BFFFE6\">" +
                            "<td>" + res_inc_per + "</td>" +
                            "<td align=\"right\">" + res_inc_cst + "</td>" +
                            "</tr>");
                   }
               } else {
                   out.println("Consulta '" + CONSULTA + "' no prevista");
               }

               out.println("</table><br><table border=0><tr>");
               out.println("<td><INPUT TYPE=\"button\" VALUE=\"Inici\" onClick=\"window.location.href='./MenuInici.html'\"></td>");
               out.println("<td></td>");
               out.println("<td><INPUT TYPE=\"button\" VALUE=\"Enrera\" onClick=\"history.go(-1);return true;\"></td>");
               out.println("</tr></table></body></html>");
               statement.close();
            } else {
               out.println("No s'ha indicat la consulta a fer. Utilitza el format URL?consulta=xxx");
            }

        } catch(Exception ex) {
            out.println("<HTML>" +
                        "<Head><Title>" +
                        "*** Error ***" +
                        "</Title></Head>\n<Body>" +
                        "<P>SQL error in doGet: " +
                        ex.getMessage() +
                        "</P></Body></HTML>");
            return;
        }
        out.close();
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        doGet(request, response);
    }

    // Adaptaci: http://www.dreamincode.net/forums/topic/14886-date-validation-using-simpledateformat/
    // date validation using SimpleDateFormat
    // it will take a string and make sure it's in the proper
    // format as defined by you, and it will also make sure that
    // it's a legal date
    public boolean isValidDate(String date)    {
        // set date format, this can be changed to whatever format
        // you want, MM-dd-yyyy, MM.dd.yyyy, dd.MM.yyyy etc.
        // you can read more about it here:
        // http://java.sun.com/j2se/1.4.2/docs/api/index.html

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        // declare and initialize testDate variable, this is what will hold
        // our converted string

        java.util.Date testDate = null;

        // we will now try to parse the string into date form
        try {
          testDate = sdf.parse(date);
        }

        // if the format of the string provided doesn't match the format we
        // declared in SimpleDateFormat() we will get an exception
        catch (ParseException e) {
          // errorMessage = "the date you provided is in an invalid date format.";
          return false;
        }

        // dateformat.parse will accept any date as long as it's in the format
        // you defined, it simply rolls dates over, for example, december 32
        // becomes jan 1 and december 0 becomes november 30
        // This statement will make sure that once the string
        // has been checked for proper formatting that the date is still the
        // date that was entered, if it's not, we assume that the date is invalid
        if (!sdf.format(testDate).equals(date)) {
          // errorMessage = "The date that you provided is invalid.";
          return false;
        }

        // if we make it to here without getting an error it is assumed that
        // the date was a valid one and that it's in the proper format
        return true;
    } // end isValidDate
}
