﻿--====================================================================
-- Funcions
--====================================================================
CREATE OR REPLACE FUNCTION gravar_log(o NAME, t TEXT, dd TEXT) RETURNS void AS $$
BEGIN
  INSERT INTO log (data, hora, usuari, operacio, taula, dades)
           VALUES (current_date, clock_timestamp(), current_user, $1, $2, $3);
END;
$$ LANGUAGE plpgsql;

--====================================================================
-- Procediments TAULA gamma
--====================================================================
CREATE OR REPLACE FUNCTION gamma_stamp() RETURNS trigger AS $gamma_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiGamma  IS NULL THEN errorFound = TRUE; errorText='codiGamma';
      ELSIF NEW.descripcio IS NULL THEN errorFound = TRUE; errorText='descripcio';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiGamma i descripcio SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;
    END IF;

    logText = ROW.codiGamma ||';'|| ROW.descripcio ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiGamma ||';'|| OLD.descripcio ||';-->;'|| logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $gamma_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS gamma_stamp ON gamma;
CREATE TRIGGER gamma_stamp BEFORE INSERT OR UPDATE OR DELETE ON gamma
   FOR EACH ROW EXECUTE PROCEDURE gamma_stamp();

--====================================================================
-- Procediments TAULA tipus_prd
--====================================================================
CREATE OR REPLACE FUNCTION tipus_prd_stamp() RETURNS trigger AS $tipus_prd_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiTipusPrd IS NULL THEN errorFound = TRUE; errorText='codiTipusPrd';
      ELSIF NEW.gamma        IS NULL THEN errorFound = TRUE; errorText='gamma';
      ELSIF NEW.descripcio   IS NULL THEN errorFound = TRUE; errorText='descripcio';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiTipusPrd, gamma i descripcio SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiGamma FROM gamma WHERE codiGamma = NEW.gamma;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant tipus_prd: gamma % no valida, no a taula gamma',
           NEW.gamma;
      END IF;
    END IF;

    logText = ROW.codiTipusPrd ||';'|| ROW.gamma ||';'|| ROW.descripcio ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiTipusPrd ||';'|| OLD.gamma ||';'|| OLD.descripcio ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $tipus_prd_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS tipus_prd_stamp ON tipus_prd;
CREATE TRIGGER tipus_prd_stamp BEFORE INSERT OR UPDATE OR DELETE ON tipus_prd
   FOR EACH ROW EXECUTE PROCEDURE tipus_prd_stamp();

--====================================================================
-- Procediments TAULA producte
--====================================================================
CREATE OR REPLACE FUNCTION producte_stamp() RETURNS trigger AS $producte_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiProducte   IS NULL THEN errorFound=TRUE; errorText='codiProducte';
      ELSIF NEW.descripcio     IS NULL THEN errorFound=TRUE; errorText='descripcio';
      ELSIF NEW.tipus          IS NULL THEN errorFound=TRUE; errorText='tipus';
      ELSIF NEW.preuCost       IS NULL THEN errorFound=TRUE; errorText='preuCost';
      ELSIF NEW.preuVenda      IS NULL THEN errorFound=TRUE; errorText='preuVenda';
      ELSIF NEW.duradaGarantia IS NULL THEN errorFound=TRUE; errorText='duradaGarantia';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiProducte, descripcio, tipus, preuCost, preuVenda';
         RAISE NOTICE '--             i duradaGarantia SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiTipusPrd FROM tipus_prd WHERE codiTipusPrd = NEW.tipus;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant producte: tipus producte % no valid, no a taula tipus_prd',
           NEW.tipus;
      END IF;

      IF NEW.preuCost >= NEW.preuVenda THEN
         RAISE EXCEPTION
           '-- Error gravant producte: preu cost (%) no pot ser > o = a preu venda (%).',
           NEW.preuCost, NEW.preuVenda;
      END IF;

    END IF;

    logText = ROW.codiProducte ||';'|| ROW.descripcio ||';'|| ROW.tipus ||';'
           || ROW.preuCost ||';'    || ROW.preuVenda ||';' || ROW.duradaGarantia ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiProducte ||';'|| OLD.descripcio ||';'|| OLD.tipus ||';'
             || OLD.preuCost ||';'    || OLD.preuVenda ||';' || OLD.duradaGarantia ||';-->'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $producte_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS producte_stamp ON producte;
CREATE TRIGGER producte_stamp BEFORE INSERT OR UPDATE OR DELETE ON producte
   FOR EACH ROW EXECUTE PROCEDURE producte_stamp();

--====================================================================
-- Procediments TAULA client
--====================================================================
CREATE OR REPLACE FUNCTION client_stamp() RETURNS trigger AS $client_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiClient IS NULL THEN errorFound=TRUE; errorText='codiClient';
      ELSIF NEW.nif        IS NULL THEN errorFound=TRUE; errorText='nif';
      ELSIF NEW.nom        IS NULL THEN errorFound=TRUE; errorText='nom';
      ELSIF NEW.adress     IS NULL THEN errorFound=TRUE; errorText='adress';
      ELSIF NEW.telefon    IS NULL THEN errorFound=TRUE; errorText='telefon';
      ELSIF NEW.descompte  IS NULL THEN errorFound=TRUE; errorText='descompte';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiClient, nif, nom, adress, telefon,';
         RAISE NOTICE '--             i descompte SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;
    END IF;

    logText = ROW.codiClient ||';'|| ROW.nif ||';'     || ROW.nom ||';'
           || ROW.adress ||';'    || ROW.telefon ||';' || ROW.descompte ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiClient ||';'|| OLD.nif ||';'     || OLD.nom ||';'
	         || OLD.adress ||';'    || OLD.telefon ||';' || OLD.descompte ||';-->;'
	         || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $client_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS client_stamp ON client;
CREATE TRIGGER client_stamp BEFORE INSERT OR UPDATE OR DELETE ON client
   FOR EACH ROW EXECUTE PROCEDURE client_stamp();

--====================================================================
-- Procediments TAULA empleat
--====================================================================
CREATE OR REPLACE FUNCTION empleat_stamp() RETURNS trigger AS $empleat_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiEmpleat IS NULL THEN errorFound=TRUE; errorText='codiEmpleat';
      ELSIF NEW.nif         IS NULL THEN errorFound=TRUE; errorText='nif';
      ELSIF NEW.nom         IS NULL THEN errorFound=TRUE; errorText='nom';
      ELSIF NEW.adress      IS NULL THEN errorFound=TRUE; errorText='adress';
      ELSIF NEW.nss         IS NULL THEN errorFound=TRUE; errorText='nss';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiEmpleat, nif, nom, adress i nss SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;
    END IF;

    logText = ROW.codiEmpleat ||';'|| ROW.nif ||';'     || ROW.nom ||';'
           || ROW.adress ||';'     || ROW.telefon ||';' || ROW.nss ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiEmpleat ||';'|| OLD.nif ||';'     || OLD.nom ||';'
	         || OLD.adress ||';'     || OLD.telefon ||';' || OLD.nss ||';-->;'
	         || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $empleat_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS empleat_stamp ON empleat;
CREATE TRIGGER empleat_stamp BEFORE INSERT OR UPDATE OR DELETE ON empleat
   FOR EACH ROW EXECUTE PROCEDURE empleat_stamp();

--====================================================================
-- Procediments TAULA venedor
--====================================================================
CREATE OR REPLACE FUNCTION venedor_stamp() RETURNS trigger AS $venedor_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiEmpleat    IS NULL THEN errorFound = TRUE; errorText='codiEmpleat';
      ELSIF NEW.comissioVendes IS NULL THEN errorFound = TRUE; errorText='comissioVendes';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiEmpleat i comissioVendes SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      IF    NEW.comissioVendes < 0 THEN
         RAISE NOTICE '-- El venedor no ha de pagar la seva feina...';
         RAISE EXCEPTION '-- Error comissio de venedor negativa = %', NEW.comissioVendes;
      ELSIF NEW.comissioVendes > 20 then
         RAISE NOTICE '-- El venedor no ha de viure nomes de les comissions...';
         RAISE EXCEPTION '-- Error comissio de venedor excessiva (max.20) = %',
           NEW.comissioVendes;
      END IF;

      SELECT INTO regFound codiEmpleat FROM empleat WHERE codiEmpleat = NEW.codiEmpleat;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant venedor: venedor % no valid, no es a taula empleat',
           NEW.codiEmpleat;
      END IF;

      SELECT INTO regFound codiEmpleat FROM enginyer WHERE codiEmpleat = NEW.codiEmpleat;
      IF FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant venedor: empleat % no pot ser venedor, es enginyer',
           NEW.codiEmpleat;
      ELSE
        SELECT INTO regFound codiEmpleat FROM agent_telefonic
              WHERE codiEmpleat = NEW.codiEmpleat;
        IF FOUND THEN
           RAISE EXCEPTION
             '-- Error gravant venedor: empleat % no pot ser venedor, es agent_telefonic',
             NEW.codiEmpleat;
        END IF;
      END IF;

    END IF;

    logText = ROW.codiEmpleat ||';'|| ROW.comissioVendes ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiEmpleat ||';'|| OLD.comissioVendes ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $venedor_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS venedor_stamp ON venedor;
CREATE TRIGGER venedor_stamp BEFORE INSERT OR UPDATE OR DELETE ON venedor
   FOR EACH ROW EXECUTE PROCEDURE venedor_stamp();

--====================================================================
-- Procediments TAULA agent_telefonic
--====================================================================
CREATE OR REPLACE FUNCTION agent_telefonic_stamp() RETURNS trigger AS $agent_telefonic_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiEmpleat        IS NULL THEN errorFound = TRUE; errorText='codiEmpleat';
      ELSIF NEW.telefonDepartament IS NULL THEN errorFound = TRUE; errorText='telefonDepartament';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiEmpleat i telefonDepartament SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiEmpleat FROM empleat WHERE codiEmpleat = NEW.codiEmpleat;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant agent_telefonic: % no valid, no es a taula empleat',
           NEW.codiEmpleat;
      END IF;

      SELECT INTO regFound codiEmpleat FROM enginyer WHERE codiEmpleat = NEW.codiEmpleat;
      IF FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant agent_telefonic: empleat % no pot ser agent_telefonic, es enginyer',
           NEW.codiEmpleat;
      ELSE
        SELECT INTO regFound codiEmpleat FROM venedor WHERE codiEmpleat = NEW.codiEmpleat;
        IF FOUND THEN
           RAISE EXCEPTION
             '-- Error gravant agent_telefonic: empleat % no pot ser agent_telefonic, es venedor',
             NEW.codiEmpleat;
        END IF;
      END IF;

    END IF;

    logText = ROW.codiEmpleat ||';'|| ROW.telefonDepartament ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiEmpleat ||';'|| OLD.telefonDepartament ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $agent_telefonic_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS agent_telefonic_stamp ON agent_telefonic;
CREATE TRIGGER agent_telefonic_stamp BEFORE INSERT OR UPDATE OR DELETE ON agent_telefonic
   FOR EACH ROW EXECUTE PROCEDURE agent_telefonic_stamp();

--====================================================================
-- Procediments TAULA enginyer
--====================================================================
CREATE OR REPLACE FUNCTION enginyer_stamp() RETURNS trigger AS $enginyer_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiEmpleat  IS NULL THEN errorFound = TRUE; errorText='codiEmpleat';
      ELSIF NEW.especialitat IS NULL THEN errorFound = TRUE; errorText='especialitat';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiEmpleat i especialitat SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiEmpleat FROM empleat WHERE codiEmpleat = NEW.codiEmpleat;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant enginyer: % no valid, no es a taula empleat',
           NEW.codiEmpleat;
      END IF;

      SELECT INTO regFound codiEmpleat FROM agent_telefonic WHERE codiEmpleat = NEW.codiEmpleat;
      IF FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant enginyer: empleat % no pot ser enginyer, es agent_telefonic',
           NEW.codiEmpleat;
      ELSE
        SELECT INTO regFound codiEmpleat FROM venedor WHERE codiEmpleat = NEW.codiEmpleat;
        IF FOUND THEN
           RAISE EXCEPTION
             '-- Error gravant enginyer: empleat % no pot ser enginyer, es venedor',
             NEW.codiEmpleat;
        END IF;
      END IF;

    END IF;

    logText = ROW.codiEmpleat ||';'|| ROW.especialitat ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiEmpleat ||';'|| OLD.especialitat ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $enginyer_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS enginyer_stamp ON enginyer;
CREATE TRIGGER enginyer_stamp BEFORE INSERT OR UPDATE OR DELETE ON enginyer
   FOR EACH ROW EXECUTE PROCEDURE enginyer_stamp();

--====================================================================
-- Procediments TAULA especialitzacio
--====================================================================
CREATE OR REPLACE FUNCTION especialitzacio_stamp() RETURNS trigger AS $especialitzacio_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.enginyer   IS NULL THEN errorFound = TRUE; errorText='enginyer';
      ELSIF NEW.gamma      IS NULL THEN errorFound = TRUE; errorText='gamma';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: enginyer i gamma SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiEmpleat FROM enginyer WHERE codiEmpleat = NEW.enginyer;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant especialitzacio: enginyer % no valid, no es a taula enginyer',
           NEW.enginyer;
      END IF;

      SELECT INTO regFound codiGamma FROM gamma WHERE codiGamma = NEW.gamma;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant especialitzacio: gamma % no valida, no es a taula gamma',
           NEW.gamma;
      END IF;
    END IF;

    logText = ROW.enginyer ||';'|| ROW.gamma ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.enginyer ||';'|| OLD.gamma ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $especialitzacio_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS especialitzacio_stamp ON especialitzacio;
CREATE TRIGGER especialitzacio_stamp BEFORE INSERT OR UPDATE OR DELETE ON especialitzacio
   FOR EACH ROW EXECUTE PROCEDURE especialitzacio_stamp();

--====================================================================
-- Procediments TAULA factura
--====================================================================
CREATE OR REPLACE FUNCTION factura_stamp() RETURNS trigger AS $factura_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiFactura IS NULL THEN errorFound = TRUE; errorText='codiFactura';
      ELSIF NEW.venedor     IS NULL THEN errorFound = TRUE; errorText='venedor';
      ELSIF NEW.data        IS NULL THEN errorFound = TRUE; errorText='data';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiFactura, venedor i data SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiEmpleat FROM venedor WHERE codiEmpleat = NEW.venedor;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant factura: % no valid, no es a taula venedor',
           NEW.venedor;
      END IF;

    END IF;

    logText = ROW.codiFactura ||';'|| ROW.venedor ||';'|| ROW.data ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiFactura ||';'|| OLD.venedor ||';'|| OLD.data ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $factura_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS factura_stamp ON factura;
CREATE TRIGGER factura_stamp BEFORE INSERT OR UPDATE OR DELETE ON factura
   FOR EACH ROW EXECUTE PROCEDURE factura_stamp();

--====================================================================
-- Procediments TAULA adquisicio
--====================================================================
CREATE OR REPLACE FUNCTION adquisicio_stamp() RETURNS trigger AS $adquisicio_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.producte   IS NULL THEN errorFound = TRUE; errorText='producte';
      ELSIF NEW.factura    IS NULL THEN errorFound = TRUE; errorText='factura';
      ELSIF NEW.client     IS NULL THEN errorFound = TRUE; errorText='client';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: producte, factura i client SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiProducte FROM producte WHERE codiProducte = NEW.producte;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: producte % no valid, no es a taula producte',
           NEW.producte;
      END IF;

      SELECT INTO regFound codiFactura FROM factura WHERE codiFactura = NEW.factura;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: factura % no valida, no es a taula factura',
           NEW.factura;
      END IF;

      SELECT INTO regFound codiClient FROM client WHERE codiClient = NEW.client;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: client % no valida, no es a taula client',
           NEW.client;
      END IF;
    END IF;

    logText = ROW.producte ||';'|| ROW.factura ||';'|| ROW.client ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.producte ||';'|| OLD.factura ||';'|| OLD.client ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $adquisicio_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS adquisicio_stamp ON adquisicio;
CREATE TRIGGER adquisicio_stamp BEFORE INSERT OR UPDATE OR DELETE ON adquisicio
   FOR EACH ROW EXECUTE PROCEDURE adquisicio_stamp();

--====================================================================
-- Procediments TAULA tipus_inc
--====================================================================
CREATE OR REPLACE FUNCTION tipus_inc_stamp() RETURNS trigger AS $tipus_inc_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiTipusInc      IS NULL THEN errorFound = TRUE; errorText='codiTipusInc';
      ELSIF NEW.descripcio        IS NULL THEN errorFound = TRUE; errorText='descripcio';
      ELSIF NEW.garantiaAplicable IS NULL THEN errorFound = TRUE; errorText='garantiaAplicable';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiTipusInc, descripcio i garantiaAplicable SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      IF NEW.garantiaAplicable != 'S' AND NEW.garantiaAplicable != 'N' THEN
         RAISE NOTICE '-- Cal indicar si el tipus de incidencia entrara en garantia o no';
         RAISE NOTICE '-- Valors possibles S o N';
         RAISE EXCEPTION '-- Error: valor de garantiaAplicable = %', NEW.garantiaAplicable;
      END IF;

    END IF;

    logText = ROW.codiTipusInc ||';'|| ROW.descripcio ||';'|| ROW.garantiaAplicable ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.codiTipusInc ||';'|| OLD.descripcio ||';'|| OLD.garantiaAplicable ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $tipus_inc_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS tipus_inc_stamp ON tipus_inc;
CREATE TRIGGER tipus_inc_stamp BEFORE INSERT OR UPDATE OR DELETE ON tipus_inc
   FOR EACH ROW EXECUTE PROCEDURE tipus_inc_stamp();

--====================================================================
-- Procediments TAULA incidencia
--====================================================================
CREATE OR REPLACE FUNCTION incidencia_stamp() RETURNS trigger AS $incidencia_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    logText2 TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiIncidencia  IS NULL THEN errorFound = TRUE; errorText='codiIncidencia';
      ELSIF NEW.producte        IS NULL THEN errorFound = TRUE; errorText='producte';
      ELSIF NEW.factura         IS NULL THEN errorFound = TRUE; errorText='factura';
      ELSIF NEW.tipusIncidencia IS NULL THEN errorFound = TRUE; errorText='tipusIncidencia';
      ELSIF NEW.dataObertura    IS NULL THEN errorFound = TRUE; errorText='dataObertura';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiIncidencia, producte, factura, tipusIncidencia,';
         RAISE NOTICE '--             i dataObertura SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiProducte FROM producte WHERE codiProducte = NEW.producte;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: producte % no valid, no es a taula producte',
           NEW.producte;
      END IF;

      SELECT INTO regFound codiFactura FROM factura WHERE codiFactura = NEW.factura;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: factura % no valida, no es a taula factura',
           NEW.factura;
      END IF;

      SELECT INTO regFound codiTipusInc FROM tipus_inc WHERE codiTipusInc = NEW.tipusIncidencia;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: tipusIncidencia % no valida, no es a taula tipus_inc',
           NEW.tipusIncidencia;
      END IF;

      IF NEW.dataTancament IS NOT NULL THEN
         IF NEW.dataTancament < NEW.dataObertura THEN
            RAISE EXCEPTION '-- Error: dataTancament (%) < dataObertura (%)',
                  NEW.dataTancament, NEW.dataObertura;
         END IF;
      END IF;

      SELECT INTO regFound count(*) FROM adquisicio WHERE producte = NEW.producte
             AND factura = NEW.factura;
      IF regFound != '1' THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: no hi ha adquisicio amb producte = %, i factura = %',
           NEW.producte, NEW.factura;
      END IF;

      SELECT INTO regFound data FROM factura WHERE codiFactura = NEW.factura;
      IF NEW.dataObertura < to_date(regFound,'YYYY-MM-DD') THEN
         RAISE EXCEPTION
           '-- Error gravant adquisicio: dataObertura incidencia (%) < data factura (%)',
           NEW.dataObertura, regFound;
      END IF;

    END IF;

    logText = ROW.codiIncidencia  ||';'|| ROW.producte     ||';'|| ROW.factura       ||';'
           || ROW.tipusIncidencia ||';'|| ROW.dataObertura ||';';
    IF  ROW.dataTancament IS NOT NULL THEN
       logText = '' || logText || ROW.dataTancament ||';';
    END IF;

    IF (TG_OP = 'UPDATE') THEN
      logText2 = OLD.codiIncidencia  ||';'|| OLD.producte     ||';'|| OLD.factura       ||';'
              || OLD.tipusIncidencia ||';'|| OLD.dataObertura ||';';
      IF  OLD.dataTancament IS NOT NULL THEN logText2 = logText2 || OLD.dataTancament ||';'; END IF;
      logText = logText2 ||'-->;'|| logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $incidencia_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS incidencia_stamp ON incidencia;
CREATE TRIGGER incidencia_stamp BEFORE INSERT OR UPDATE OR DELETE ON incidencia
   FOR EACH ROW EXECUTE PROCEDURE incidencia_stamp();

--====================================================================
-- Procediments TAULA comunicacio
--====================================================================
CREATE OR REPLACE FUNCTION comunicacio_stamp() RETURNS trigger AS $comunicacio_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.incidencia     IS NULL THEN errorFound = TRUE; errorText='incidencia';
      ELSIF NEW.client         IS NULL THEN errorFound = TRUE; errorText='client';
      ELSIF NEW.agentTelefonic IS NULL THEN errorFound = TRUE; errorText='agentTelefonic';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: incidencia, client i agentTelefonic SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiIncidencia FROM incidencia WHERE codiIncidencia = NEW.incidencia;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant comunicacio: incidencia % no valida, no es a taula incidencia',
           NEW.incidencia;
      END IF;

      SELECT INTO regFound codiClient FROM client WHERE codiClient = NEW.client;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant comunicacio: client % no valida, no es a taula client',
           NEW.client;
      END IF;

      SELECT INTO regFound codiEmpleat FROM agent_Telefonic WHERE codiEmpleat = NEW.agentTelefonic;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant comunicacio: agentTelefonic % no valid, no es a taula agent_telefonic',
           NEW.agentTelefonic;
      END IF;

      SELECT INTO regFound client FROM adquisicio
             WHERE producte = (SELECT producte FROM incidencia WHERE codiIncidencia = NEW.incidencia)
             AND   factura  = (SELECT factura  FROM incidencia WHERE codiIncidencia = NEW.incidencia);
      IF regFound != NEW.client THEN
         RAISE EXCEPTION
           '-- Error gravant comunicacio: error client: qui va comprar (%) no es qui comunica inc.(%)',
           regFound, NEW.client;
      END IF;

    END IF;

    logText = ROW.incidencia ||';'|| ROW.client ||';'|| ROW.agentTelefonic ||';';
    IF (TG_OP = 'UPDATE') THEN
      logText = OLD.incidencia ||';'|| OLD.client ||';'|| OLD.agentTelefonic ||';-->;'
             || logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $comunicacio_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS comunicacio_stamp ON comunicacio;
CREATE TRIGGER comunicacio_stamp BEFORE INSERT OR UPDATE OR DELETE ON comunicacio
   FOR EACH ROW EXECUTE PROCEDURE comunicacio_stamp();

--====================================================================
-- Procediments TAULA accio
--====================================================================
CREATE OR REPLACE FUNCTION accio_stamp() RETURNS trigger AS $accio_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    logText2 TEXT;
    regFound TEXT;
    row RECORD;
  BEGIN
    IF (TG_OP = 'DELETE') THEN
      ROW = OLD;
    ELSE  -- 'INSERT' & 'UPDATE'
      ROW = NEW;
      -- Comprovar que els valors son correctes
      errorFound = FALSE;
      IF    NEW.codiAccio  IS NULL THEN errorFound = TRUE; errorText='codiAccio';
      ELSIF NEW.incidencia IS NULL THEN errorFound = TRUE; errorText='incidencia';
      ELSIF NEW.enginyer   IS NULL THEN errorFound = TRUE; errorText='enginyer';
      ELSIF NEW.dataInici  IS NULL THEN errorFound = TRUE; errorText='dataInici';
      END IF;
      IF errorFound THEN
         RAISE NOTICE '-- ELS VALORS: codiAccio, incidencia, enginyer i dataInici';
         RAISE NOTICE '--             SON OBLIGATORIS.';
         RAISE NOTICE '-- OPERACIO  : % a %', TG_OP, TG_TABLE_NAME;
         RAISE NOTICE '-- REGISTRE  : %', NEW;
         RAISE EXCEPTION '-- Error trobat valor NUL a %', errorText;
      END IF;

      SELECT INTO regFound codiIncidencia FROM incidencia WHERE codiIncidencia = NEW.incidencia;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant accio: incidencia % no valida, no es a taula incidencia',
           NEW.incidencia;
      END IF;

      SELECT INTO regFound codiEmpleat FROM enginyer WHERE codiEmpleat = NEW.enginyer;
      IF NOT FOUND THEN
         RAISE EXCEPTION
           '-- Error gravant accio: enginyer % no valid, no es a taula enginyer',
           NEW.enginyer;
      END IF;

      IF NEW.dataFi IS NOT NULL THEN
         IF NEW.dataFi < NEW.dataInici THEN
            RAISE EXCEPTION '-- Error gravant accio: dataFi (%) < dataInici (%)',
              NEW.dataFi, NEW.dataInici;
         END IF;
      END IF;

      IF NEW.cost < 0 THEN
         RAISE EXCEPTION '-- Error gravant accio: cost accio (%) < 0', NEW.cost;
      END IF;

      SELECT INTO regFound dataObertura FROM incidencia WHERE codiIncidencia = NEW.incidencia;
      IF NEW.dataInici < to_date(regFound,'YYYY-MM-DD') THEN
         RAISE EXCEPTION
           '-- Error gravant accio: dataInici accio (%) < data inici incidencia (%)',
           NEW.dataInici, regFound;
      END IF;

      SELECT INTO regFound dataTancament FROM incidencia WHERE codiIncidencia = NEW.incidencia;
      IF regFound IS NOT NULL THEN
         IF NEW.dataInici > to_date(regFound,'YYYY-MM-DD') THEN
           RAISE EXCEPTION
             '-- Error gravant accio: dataInici accio (%) > data tancament incidencia (%)',
             NEW.dataInici, regFound;
         ELSIF NEW.dataFi > to_date(regFound,'YYYY-MM-DD') THEN
           RAISE EXCEPTION
             '-- Error gravant accio: dataFi accio (%) > data tancament incidencia (%)',
             NEW.dataFi, regFound;
         END IF;
      END IF;
    END IF;

    logText = ROW.codiAccio ||';'|| ROW.incidencia ||';'|| ROW.enginyer ||';'
           || ROW.dataInici ||';';
    IF   ROW.dataFi IS NOT NULL THEN logText = logText || ROW.dataFi ||';';
    ELSE logText = logText ||';';
    END IF;
    logText = logText || ROW.cost ||';';

    IF (TG_OP = 'UPDATE') THEN
      logText2 = OLD.codiAccio ||';'|| OLD.incidencia ||';'|| OLD.enginyer ||';'
              || OLD.dataInici ||';';
      IF  OLD.dataFi IS NOT NULL THEN logText2 = logText2 || OLD.dataFi ||';';
      ELSE logText2 = logText2 ||';';
      END IF;
      logText = logText2 ||'-->;'|| logText;
    END IF;
    logCall = gravar_log(TG_OP, TG_TABLE_NAME, logText);
    RETURN ROW;
  END;
  $accio_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS accio_stamp ON accio;
CREATE TRIGGER accio_stamp BEFORE INSERT OR UPDATE OR DELETE ON accio
   FOR EACH ROW EXECUTE PROCEDURE accio_stamp();
