--CREATE DATABASE electroDWH

----------------------------------------------------------------------
-- Neteja de taules
----------------------------------------------------------------------
DROP TABLE IF EXISTS DWH_res;
DROP TABLE IF EXISTS DWH_acc;
DROP TABLE IF EXISTS DWH_inc;
DROP TABLE IF EXISTS DWH_prd;

----------------------------------------------------------------------
-- TAULA DWH_prd -- producte DWH --
----------------------------------------------------------------------
CREATE TABLE DWH_prd (
  DWH_prd_codi  CHAR(12)    CONSTRAINT PK_DWH_prd       PRIMARY KEY,
  DWH_prd_descr VARCHAR(32) CONSTRAINT NN_DWH_prd_descr NOT NULL,
  DWH_prd_tipus VARCHAR(32) CONSTRAINT NN_DWH_prd_tipus NOT NULL,
  DWH_prd_gamma VARCHAR(16) CONSTRAINT NN_DWH_prd_gamma NOT NULL,
  DWH_prd_data  DATE        CONSTRAINT NN_DWH_prd_data  NOT NULL,
  DWH_prd_tmGar INTEGER     CONSTRAINT NN_DWH_prd_tmGar NOT NULL
);

----------------------------------------------------------------------
-- TAULA DWH_inc -- incidencia DWH --
----------------------------------------------------------------------
CREATE TABLE DWH_inc (
  DWH_inc_codi  CHAR(12)    CONSTRAINT PK_DWH_inc       PRIMARY KEY,
  DWH_inc_prod  CHAR(12)    CONSTRAINT FK_DWH_inc_prod  REFERENCES DWH_prd(DWH_prd_codi)
                            CONSTRAINT NN_DWH_inc_prod  NOT NULL,
  DWH_inc_tipus VARCHAR(32) CONSTRAINT NN_DWH_inc_tipus NOT NULL,
  DWH_inc_dtIni DATE        CONSTRAINT NN_DWH_inc_dtIni NOT NULL,
  DWH_inc_dtFi  DATE,
  DWH_inc_InGar CHAR(1)     CONSTRAINT NN_DWH_inc_InGar NOT NULL,
                            CONSTRAINT CH_DWH_inc_InGar
                                 CHECK(DWH_inc_InGar IN ('S', 'N'))
);

----------------------------------------------------------------------
-- TAULA DWH_acc -- accio DWH --
----------------------------------------------------------------------
CREATE TABLE DWH_acc (
  DWH_acc_codi  CHAR(12)    CONSTRAINT PK_DWH_acc       PRIMARY KEY,
  DWH_acc_incid CHAR(12)    CONSTRAINT FK_DWH_acc_incid REFERENCES DWH_inc(DWH_inc_codi)
                            CONSTRAINT NN_DWH_acc_incid NOT NULL,
  DWH_acc_cost  NUMERIC     CONSTRAINT NN_DWH_acc_cost  NOT NULL
);

----------------------------------------------------------------------
-- TAULA DWH_res -- resultats DWH -- Fet --
----------------------------------------------------------------------
CREATE TABLE DWH_res (
  DWH_res_prod  CHAR(12)    CONSTRAINT FK_DWH_res_prod  REFERENCES DWH_prd(DWH_prd_codi)
                            CONSTRAINT NN_DWH_res_prod  NOT NULL,
  DWH_res_incid CHAR(12)    CONSTRAINT FK_DWH_res_inci  REFERENCES DWH_inc(DWH_inc_codi)
                            CONSTRAINT NN_DWH_res_inci  NOT NULL,
  DWH_res_accio CHAR(12)    CONSTRAINT FK_DWH_res_accio REFERENCES DWH_acc(DWH_acc_codi)
                            CONSTRAINT NN_DWH_res_accio NOT NULL,
                            CONSTRAINT PK_DWH_res       PRIMARY KEY(DWH_res_prod,
                                                            DWH_res_incid, DWH_res_accio)
);
