--====================================================================
-- Procediments TAULA DWH_acc
----------------------------------------------------------------------
-- Quan s'actualitzi la taula,
-- es mantindra actualitzada la taula de resultats DWH_res
--====================================================================
CREATE OR REPLACE FUNCTION DWH_acc_stamp() RETURNS trigger AS $DWH_acc_stamp$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
  BEGIN
    -- Comprovem que la incidencia existeix
    SELECT INTO regFound DWH_inc_codi FROM DWH_inc WHERE DWH_inc_codi = NEW.DWH_acc_incid;
    IF NOT FOUND THEN
       RAISE EXCEPTION
         '-- Error gravant DWH_acc: incidencia % no valida, no a taula DWH_inc',
         NEW.DWH_acc_incid;
    END IF;

    IF (TG_OP = 'UPDATE') THEN
      -- Actualitzem el registre a la taula de resultats
      UPDATE DWH_res
         SET DWH_res_prod  = (SELECT DWH_inc_prod
                                FROM DWH_inc
                               WHERE DWH_inc_codi = NEW.DWH_acc_incid),
             DWH_res_incid = NEW.DWH_acc_incid,
             DWH_res_accio = NEW.DWH_acc_codi
       WHERE DWH_res_prod = (SELECT DWH_inc_prod FROM DWH_inc
                              WHERE DWH_inc_codi = OLD.DWH_acc_incid)
         AND DWH_res_incid = OLD.DWH_acc_incid
         AND DWH_res_accio = OLD.DWH_acc_codi;
    ELSE -- (TG_OP = 'INSERT') THEN
      -- Gravem el registre a la taula de resultats
      INSERT INTO DWH_res (DWH_res_prod, DWH_res_incid, DWH_res_accio)
                   VALUES ((SELECT DWH_inc_prod
                              FROM DWH_inc
                             WHERE DWH_inc_codi = NEW.DWH_acc_incid),
                           NEW.DWH_acc_incid, NEW.DWH_acc_codi);
    END IF;

    RETURN NEW;
  END;
  $DWH_acc_stamp$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS DWH_acc_stamp ON DWH_acc;
CREATE TRIGGER DWH_acc_stamp AFTER INSERT OR UPDATE ON DWH_acc
   FOR EACH ROW EXECUTE PROCEDURE DWH_acc_stamp();
--====================================================================
CREATE OR REPLACE FUNCTION DWH_acc_delete() RETURNS trigger AS $DWH_acc_delete$
  DECLARE
    errorFound BOOLEAN;
    errorText TEXT;
    logCall TEXT;
    logText TEXT;
    regFound TEXT;
  BEGIN
    -- Comprovem que la incidencia existeix
    SELECT INTO regFound DWH_inc_codi FROM DWH_inc WHERE DWH_inc_codi = OLD.DWH_acc_incid;
    IF NOT FOUND THEN
       RAISE EXCEPTION
         '-- Error gravant DWH_acc: incidencia % no valida, no a taula DWH_inc',
         OLD.DWH_acc_incid;
    END IF;

    -- Esborrem el registre a la taula de resultats
    DELETE FROM DWH_res
          WHERE DWH_res_prod = (SELECT DWH_inc_prod FROM DWH_inc
                                 WHERE DWH_inc_codi = OLD.DWH_acc_incid)
            AND DWH_res_incid = OLD.DWH_acc_incid
            AND DWH_res_accio = OLD.DWH_acc_codi;

    RETURN OLD;
  END;
  $DWH_acc_delete$ LANGUAGE plpgsql;
----------------------------------------------------------------------
DROP TRIGGER IF EXISTS DWH_acc_delete ON DWH_acc;
CREATE TRIGGER DWH_acc_delete BEFORE DELETE ON DWH_acc
   FOR EACH ROW EXECUTE PROCEDURE DWH_acc_delete();
--====================================================================
